/**
 * BSDialog4
 * @version v0.1.6.015 (2023/12/08 22:04)
 */
class BSDialog4 {
  constructor() {
	}


	get Options() {
		return {
			ShowModal: {
				ID: null,
				Title: "",
				Message: "",
				URL: null,
				Size: "md",
				Colour: "secondary",
				ShowFooter: true,
				EasyClose: true
			},
			ShowPrompt: {
				Type: "button",
				Title: "",
				Message: "",
				Size: "md",
				EasyClose: true,
				Buttons: [
					{ Label: "Yes", Value: "Yes", Colour: "primary" },
					{ Label: "No", Value: "No", Colour: "secondary" },
					{ Label: "Cancel", Value: "Cancel", Colour: "secondary" }
				],
				Textbox: {
					Label: "",
					LabelSize: 4,
					Placeholder: "",
					Value: "",
					BoxSize: 8
				}
			},
			UpdateModal: {
				ID: null,
				Title: null,
				Body: null,
				URL: null,
				Footer: null,
				Size: null
			}
		}
	}

	get #prefix() {
		return "bsdia4_";
	}

	get #body() {
		return document.getElementsByTagName("body")[0];
	}

	get #sizeList() {
		return [
			"modal-sm",
			"modal-md",
			"modal-lg",
			"modal-xl",
			"modal-xxl"
		];
	}


	async Show(options) {
		const a = this;
		const _options = Object.assign(a.Options.ShowModal, options);

		a.addModal(_options.ID, _options.Title, _options.Size, _options.ShowFooter, _options.Colour);

		$("#" + a.#prefix + _options.ID).modal({
			backdrop: _options.EasyClose,
			show: true
		});

		if (a.#isURL(_options.URL)) {
			const response = await a.#retrieveURL(_options.URL);

			await a.Update({ ID: _options.ID, Body: response });
		} else {
			await a.Update({ ID: _options.ID, Body: _options.Message });
		}
	}

	async Prompt(options) {
		const a = this;
		const id = Math.floor(Math.random() * 10000) + 1000;
		const _options = Object.assign(a.Options.ShowPrompt, options);

		switch (_options.Type) {
			case "textbox":
				return await a.#showTextboxPrompt(id, _options);
			case "button":
			default:
				return await a.#showButtonPrompt(id, _options);
		}
	}

	async Clear() {
		const a = this;

		document.querySelectorAll('.modal').forEach(async function(e) {
			await a.Close(e.getAttribute("id"));
		});
	}

	async Close(id) {
		const a = this;

		if (id.toString().startsWith(a.#prefix)) {
			id = id.toString().substr(a.#prefix.length);
		}

		const node = document.getElementById(a.#prefix + id);

		if (node) {
			await $(node).modal('hide');

			node.parentNode.removeChild(node);
		}

		if (document.querySelectorAll('.modal').length > 0) {
			if (!a.#body.classList.contains("modal-open")) {
				a.#body.classList.add("modal-open");

				a.#body.style.paddingRight = "17px";
			}
		}
	}

	async Pop() {
		const a = this;

		if (document.querySelectorAll('.modal').length > 0) {
			const id = document.querySelectorAll('.modal')[0].getAttribute("id");

			await a.Close(id);
		}
	}

	async Update(options) {
		const a = this;
		let _options = Object.assign(a.Options.UpdateModal, options);

		const modal = a.Find(_options.ID);
		if (modal === null) {
		 	return;
		}

		if (!a.#isNullOrWhitespace(_options.Title)) {
			modal.Title.innerHTML = _options.Title;
		}

		if (!a.#isNullOrWhitespace(_options.Body)) {
			a.#html(modal.Body, _options.Body);
		}

		if (!this.#isNullOrWhitespace(_options.Body)) {
			a.#html(modal.Body, _options.Body);
		} else {
			if (this.#isURL(_options.URL)) {
				const response = await a.#retrieveURL(_options.URL);

				await a.Update({ ID: _options.ID, Body: response });
			}
		}

		if (!a.#isNullOrWhitespace(_options.Footer)) {
			a.#html(modal.Footer, _options.Footer);
		}

		if (!a.#isNullOrWhitespace(_options.Size)) {
			a.#setSize(_options.ID, "modal-" + _options.Size);
		}

	}

	Exists(id) {
		const a = this;

		return (a.Find(id) !== null);
	}

	Find(id) {
		const a = this;
		const modal = a.#body.querySelectorAll("#" + a.#prefix + id + ".modal");

		if (!modal) {
			return null;
		}

		if (modal.length <= 0) {
			return null;
		}

		return {
			Title: modal[0].querySelectorAll(".modal-title")[0],
			Header: modal[0].querySelectorAll(".modal-header")[0],
			Body: modal[0].querySelectorAll(".modal-body")[0],
			Footer: modal[0].querySelectorAll(".modal-footer")[0],
			Close: modal[0].querySelectorAll("[data-dismiss='modal']"),
			Modal: modal[0]
		};
	}


	addModal(id, title, size, showFooter, closeColour) {
		const a = this;

		// don't allow duplicates
		let modal = a.Find(id);
		if (modal !== null) {
		 	return;
		}

		let html = "";
		html += '<div class="modal fade" style="display: none;" id="' + a.#prefix + id + '" tabindex="-1" aria-labelledby="exampleModalSmLabel" aria-hidden="true">';
		html += '  <div class="modal-dialog modal-' + size + '">';
		html += '    <div class="modal-content">';
		html += '      <div class="modal-header">';
		html += '        <span class="modal-title font-weight-bold" style="cursor: default; user-select: none;" aria-label="Title">' + title + '</span>';
		html += '        <button type="button" class="close" data-dismiss="modal" aria-label="Close" style="user-select: none;">';
		html += '          <span aria-hidden="true">&times;</span>';
		html += '        </button>';
		html += '      </div>';
		html += '      <div class="modal-body">';

		html += '<div class="row mt-3 pb-3">';
		html += '  <div class="col-12 text-center">';
		html += '    <div class="spinner-border" role="status">';
		html += '	     <span class="sr-only">Loading...</span>';
		html += '    </div>';
		html += '  </div>';
		html += '</div>';

		html += '      </div>';

		if (showFooter === true) {
			html += '      <div class="modal-footer">';
			html += '        <button type="button" class="btn btn-' + closeColour + '" data-dismiss="modal">Close</button>';
			html += '      </div>';
		}

		html += '    </div>';
		html += '  </div>';
		html += '</div>';

		a.#appendHtml(a.#body, html);

		modal = a.Find(id);
		if (modal === null) {
			return;
	 	}

		$("#" + a.#prefix + id).on('hidden.bs.modal', function (event) {
			a.Close(id);
		})

		modal.Title.addEventListener("dblclick", function(e){
			e.stopPropagation();
			e.preventDefault();

			a.#toggleSize(id);
		});

	}

	async #showButtonPrompt(id, options) {
		const a = this;

		return await new Promise(async (resolve, reject) => {
			await a.Show({
				ID: id,
				Title: options.Title,
				Message: options.Message,
				Size: options.Size,
				EasyClose: options.EasyClose
			});

			let html = '';
			options.Buttons.forEach(function(e) {
				html += '<button type="button" class="btn btn-' + e.Colour + '" data-prompt-value="' + e.Value + '">' + e.Label + '</button>';
			});

			a.Update({ ID: id, Footer: html });

			const modal = a.Find(id);
			modal.Footer.querySelectorAll("button").forEach(function(button) {
				button.addEventListener("click", function(e){
					e.stopPropagation();
					e.preventDefault();

					const value = button.getAttribute("data-prompt-value");

					a.Close(id);

					resolve(value);
				});
			});

			modal.Close.forEach(function(sender) {
				sender.addEventListener("click", function(e){
					e.stopPropagation();
					e.preventDefault();

					a.Close(id);

					resolve("");
				});
			});

		});
	}

	async #showTextboxPrompt(id, options) {
		const a = this;

		return await new Promise(async (resolve, reject) => {
			options.Buttons = [
				{ Label: "OK", Value: "", Colour: "primary" },
				{ Label: "Cancel", Value: "", Colour: "secondary" }
			];

			await a.Show({
				ID: id,
				Title: options.Title,
				Message: options.Message,
				Size: options.Size,
				EasyClose: options.EasyClose
			});

			let body = '';

			if (!a.#isNullOrWhitespace(options.Message)) {
				body += '<p>' + options.Message + '</p>';
			}

			body += '<div class="form-group row">';

			if (a.#isNullOrWhitespace(options.Textbox.Label) || (options.Textbox.LabelSize <= 0)) {
				body += '<div class="col-sm-12">';
				body += '<input type="text" class="form-control" id="textbox' + id + '" placeholder="' + options.Textbox.Placeholder + '" value="' + options.Textbox.Value + '">';
				body += '</div>';
			} else {
				body += '<label for="textbox' + id + '" class="col-sm-' + options.Textbox.LabelSize + ' col-form-label">' + options.Textbox.Label + '</label>';
				body += '<div class="col-sm-' + options.Textbox.BoxSize + '">';
				body += '<input type="text" class="form-control" id="textbox' + id + '" placeholder="' + options.Textbox.Placeholder + '" value="' + options.Textbox.Value + '">';
				body += '</div>';
			}

			body += '</div>';

			let footer = '';
			options.Buttons.forEach(function(e) {
				footer += '<button type="button" class="btn btn-' + e.Colour + '" data-prompt-value="' + e.Value + '">' + e.Label + '</button>';
			});

			a.Update({
				ID: id,
				Body: body,
				Footer: footer
			});

			const modal = a.Find(id);
			const buttons = modal.Footer.querySelectorAll("button");

			buttons[0].addEventListener("click", function(e){
				e.stopPropagation();
				e.preventDefault();

				const value = modal.Body.querySelectorAll("input")[0].value;

				a.Close(id);

				resolve(value);
			});

			buttons[1].addEventListener("click", function(e){
				e.stopPropagation();
				e.preventDefault();

				a.Close(id);

				resolve("");
			});

			modal.Close.forEach(function(sender) {
				sender.addEventListener("click", function(e){
					e.stopPropagation();
					e.preventDefault();

					a.Close(id);

					resolve("");
				});
			});

		});
	}

	#appendHtml(el, html) {
		let node = document.createElement('template');
		node.innerHTML = html;

		node = node.content.firstChild;

		el.appendChild(node);
	}

	#html(el, newHtml) {
		if (jQuery) {
			jQuery(el).html(newHtml);
		} else {
			el.innerHTML = newHtml;
		}
	}

	#isNullOrWhitespace(e) {
		if (typeof (e) == "undefined") {
			return true;
		}

		if (e == null) {
			return true;
		}

		if (e == false) {
			return true;
		}

		return (e.trim().length <= 0);
	}

	async #retrieveURL(url) {
		return await new Promise(async (resolve) => {
			await fetch(url, {
				cache: 'no-cache',
				credentials: 'same-origin'
			}).then(data => data.text()).then(data => {
				resolve(data);
			}).catch((error) => {
				resolve("Error: " + error);
			});
		});
	}

	#isURL(value) {
		const a = this;

		if (a.#isNullOrWhitespace(value)) {
			return false;
		}

		if (value.startsWith("http://") || value.startsWith("https://") || value.startsWith("/")) {
			return true;
		}

		return false;
	}

	#setSize(id, size) {
		const a = this;

		const modal = a.Find(id);
		if (modal === null) {
		 	return;
		}

		let modalDialog = modal.Modal.querySelectorAll(".modal-dialog")[0];

		for (let i=0; i<a.#sizeList.length; i++) {
			modalDialog.classList.remove(a.#sizeList[i]);
		}

		modalDialog.classList.add(size);
	}

	#toggleSize(id) {
		const a = this;

		const modal = a.Find(id);
		if (modal === null) {
		 	return;
		}

		let modalDialog = modal.Modal.querySelectorAll(".modal-dialog")[0];

		let pos = 0;
		for (let i=0; i<a.#sizeList.length; i++) {
			if (modalDialog.classList.contains(a.#sizeList[i])) {
				modalDialog.classList.remove(a.#sizeList[i]);
				pos = i;
				break;
			}
		}

		pos++;

		if (pos > (a.#sizeList.length - 1)) {
			pos = 0;
		}

		modalDialog.classList.add(a.#sizeList[pos]);
	}

}


var BSDialog = new BSDialog4();